/*
                   y
                   |
                   |
                   | v5
                 3 +-----------------+ v6=(3,3,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
        v4 +-----------------+ v3    |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v7                 |
           |    v8 +---------|-------+------> x           |
           |      /          |      /3                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         3 +-----------------+                         /
         / v1=(0,0,3)        v2=(3,0,3)              z
       /
     /
    z
*/
import java.awt.Color;

public class MainCubes_v2
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      Model model1 = new Model();

      // Add the geometry to the Model
      // NOTICE that this cube is a "unit cube",
      // all the sides have length 1.
      Vertex v1 = new Vertex(0.0, 0.0, 1.0);
      Vertex v2 = new Vertex(1.0, 0.0, 1.0);
      Vertex v3 = new Vertex(1.0, 1.0, 1.0);
      Vertex v4 = new Vertex(0.0, 1.0, 1.0);
      Vertex v5 = new Vertex(0.0, 1.0, 0.0);
      Vertex v6 = new Vertex(1.0, 1.0, 0.0);
      Vertex v7 = new Vertex(1.0, 0.0, 0.0);
      Vertex v8 = new Vertex(0.0, 0.0, 0.0);

      model1.primitiveList.add( new LineSegment(v1, v2) ); // front face
      model1.primitiveList.add( new LineSegment(v2, v3) ); // front face
      model1.primitiveList.add( new LineSegment(v3, v4) ); // front face
      model1.primitiveList.add( new LineSegment(v4, v1) ); // front face
      model1.primitiveList.add( new LineSegment(v8, v7) ); // back face
      model1.primitiveList.add( new LineSegment(v7, v6) ); // back face
      model1.primitiveList.add( new LineSegment(v6, v5) ); // back face
      model1.primitiveList.add( new LineSegment(v5, v8) ); // back face
      model1.primitiveList.add( new LineSegment(v1, v8) ); // bottom face
      model1.primitiveList.add( new LineSegment(v2, v7) ); // bottom face
      model1.primitiveList.add( new LineSegment(v4, v5) ); // top face
      model1.primitiveList.add( new LineSegment(v3, v6) ); // top face

      // Make two copies of model1
      Model model2 = new Model(model1);
      Model model3 = new Model(model1);
      for (Primitive p : model1.primitiveList)
      {
         model2.addPrimitive(p);  // we are NOT making a copy of the line segment
         model3.addPrimitive(p);
      }

      // Give each Model a model matrix that places the model in the scene.
      model1.modelScale(3.0, 3.0, 3.0);      // make this cube appear larger

      model2.modelTranslate(4.0, 0.0, 0.0);  // move this cube

      model3.modelTranslate(5.0, 0.0, 0.0);  // try reversing the order of
      model3.modelScale(3.0, 3.0, 3.0);      // these two transformations


      // Give each Model a color.
      model1.setColor(Color.green);  // this DOES NOT work!
      model2.setColor(Color.blue);
      model3.setColor(Color.red);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Models to the Scene.
      scene.modelList.add(model1);
      scene.modelList.add(model2);
      scene.modelList.add(model3);

      // Create a Camera object.
      Camera camera = new Camera();

      // Give the Camera a view matrix.
      camera.view2Identity();

      // Set up the camera's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);
/*
      // Set up the camera's view frustum.
      double left   = -1.0;
      double right  =  1.0;
      double top    =  1.0;
      double bottom = -1.0;
      double near   =  1.0;
      double far    =  20.0;
      camera.projFrustum(left, right, bottom, top, near, far);
*/
      // Set up the camera's location and orientation.
      double eyex=0,    eyey=2,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      //System.err.println( scene.toString() );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);
      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();
      // Render our scene into the frame buffer.
      renderer.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_Cubes_v2_Frame%02d.ppm", 0) );

      // Spin the models 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the model by 10 degrees.
         scene.camera.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         renderer.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_Cubes_v2_Frame%02d.ppm", i));
      }
   }
}