/*
   Here is the "scene graph" for this example.

             mTop
           / |  | \
          /  |  |   \
        m1  m2  m3   m4
          \  |  |   /
           \ |  | /
              m0
              |
          PrismModel

*/
import java.awt.Color;

public class PrismClient1
{
   public static void main(String[] args)
   {
      double sqrt3 = Math.sqrt(3.0);

      // Create a Prism object.
      Model m0 = new PrismModel(2, Math.PI/4.0);
      m0.doLighting = false;
      // Give each triangle a random color.
      for (Triangle t : m0.triangleList)
      {
         t.v[0].setColorRandom();
         t.setColor( t.v[0].getColor() );
      }

      // Create four new Prism objects from m0.
      // right
      Model m1 = new Model();
      m1.addModel( m0 );
      m1.modelTranslate(2+0.5/sqrt3, 0, 0);
      // left
      Model m2 = new Model();
      m2.addModel( m0 );
      m2.modelTranslate(-2-0.5/sqrt3, 0, 0);
      m2.modelRotate(180, 0,0,1);
      // top
      Model m3 = new Model();
      m3.addModel( m0 );
      m3.modelRotate(90, 0,0,1);
      m3.modelTranslate(2+0.5/sqrt3, 0, 0);
      // bottom
      Model m4 = new Model();
      m4.addModel( m0 );
      m4.modelRotate(-90, 0,0,1);
      m4.modelTranslate(2+0.5/sqrt3, 0, 0);

      // Creat a top level model to hold the four sub models
      Model mTop = new Model();
      mTop.addModel(m1);
      mTop.addModel(m2);
      mTop.addModel(m3);
      mTop.addModel(m4);

      Model floor = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      floor.doLighting = false;
      floor.doBackFaceCulling = false;
      floor.modelTranslate(0, -4, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the top level model to the Scene.
      scene.modelList.add(mTop);
      scene.modelList.add(floor);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);


      // Create a rendering pipeline,
      SceneRender renderer = new SceneRender(scene, fb);  // default pipeline
      // Check the order of the pipeline stages            .
      System.err.println( renderer );

      // Render our scene into the frame buffer.
      renderer.renderScene();
      // Save the resulting image in a file.
      String basename = "PPM_PrismClient_1_Frame_";
      fb.dumpFB2File( String.format(basename + "%02d.ppm", 0) );

      // Spin the model 360 degrees arond two axes.
      for (int i = 1; i <= 72; i++)
      {
         // Rotate the top level model 5 degrees.
         mTop.model2Identity();
         mTop.modelRotate( 5*i, 1,0,0);
         mTop.modelRotate( 5*i, 0,1,0);
         mTop.modelRotate( 5*i, 0,0,1);

         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         renderer.renderScene();
         fb.dumpFB2File(String.format(basename + "%02d.ppm", i));
      }
   }
}