/*

*/
import java.awt.Color;

public class Sphere
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >= 2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", "Sphere");
         System.exit(-1);
      }

      // Create a Model object that holds the geometry.
      Model model = new SphereModel(k, n, 1);
      model.doBackFaceCulling = false;
      model.frontFacingIsCCW = true;
      model.facesHaveTwoSides = false;
      model.frontFaceColor = Color.red;
      model.backFaceColor = Color.darkGray.darker().darker();


      // Give the Model a color.
      model.setColor(new Color(255, 0, 255));

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(model);

      // Create a Camera object.
      Camera camera = new Camera();

      // Give the Camera a view matrix.
      camera.view2Identity();

      // Set up the camera's view frustum.
      double fov    = 25.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);
/*
      // Set up the camera's view frustum.
      double left   = -1.0;
      double right  =  1.0;
      double top    =  1.0;
      double bottom = -1.0;
      double near   =  1.0;
      double far    =  20.0;
      camera.projFrustum(left, right, bottom, top, near, far);
*/
      // Set up the camera's location and orientation.
      double eyex=0,    eyey=2,    eyez=5;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 1200;
      int vp_height =  400;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.black);
      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      // Spin the model 360 degrees.
      for (int i = 0; i <= 72; i++)
      {
         // Set a viewport within the framebuffer.
         fb.setViewport(0, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a background color.
         fb.clearVP(java.awt.Color.black);
         model.doBackFaceCulling = false;
         model.facesHaveTwoSides = false;
         // Render again.
         renderer.renderScene(scene, fb);

         // Set a viewport within the framebuffer.
         fb.setViewport(400, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a background color.
         fb.clearVP(java.awt.Color.black);
         model.doBackFaceCulling = true;
         model.facesHaveTwoSides = false;
         // Render again.
         renderer.renderScene(scene, fb);

         // Set a viewport within the framebuffer.
         fb.setViewport(800, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a background color.
         fb.clearVP(java.awt.Color.black);
         model.doBackFaceCulling = false;
         model.facesHaveTwoSides = true;
         // Render again.
         renderer.renderScene(scene, fb);

         // save the framebuffer
         fb.dumpFB2File(String.format("PPM_Sphere_Frame%02d.ppm", i));

         // Rotate the model by 5 degrees.
         scene.camera.viewRotate(5, 0, 1, 0);
      }
   }
}