/*

*/
import java.awt.Color;

public class Triangle
{
   public static void main(String[] args)
   {
      // Create a TriangleModel object.
      Model model = new TriangleModel(3, -3, 3);

      // Give each vertex an individual color.
      model.primitiveList.get(0).vertexList.get(0).setColor(new Color(255, 0, 0));
      model.primitiveList.get(0).vertexList.get(1).setColor(new Color(0, 255, 0));
      model.primitiveList.get(0).vertexList.get(2).setColor(new Color(0, 0, 255));

      model.doBackFaceCulling = true;
      model.frontFacingIsCCW = true;
      model.facesHaveTwoSides = false;
      model.frontFaceColor = Color.blue;
      model.backFaceColor = Color.green;

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.modelList.add(model);

      // Create a Camera object.
      Camera camera = new Camera();
/*
      // Set up a othogonal camera view frustum.
      double left   = -3.0;
      double right  =  3.0;
      double top    =  3.0;
      double bottom = -3.0;
      double near   =  2.0;
      double far    =  20.0;
      camera.projOrtho(left, right, bottom, top, near, far);
*/
      // Set up an perspective camera view frustum.
      double left   = -1.0;
      double right  =  1.0;
      double top    =  1.0;
      double bottom = -1.0;
      double near   =  2.0;
      double far    =  20.0;
      camera.projFrustum(left, right, bottom, top, near, far);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=6;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 1200;
      int vp_height =  400;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.black);
      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      System.err.println( renderer.toString() );
      System.err.println( scene.toString() );

      // Spin the model 360 degrees.
      for (int i = 0; i <= 72; i++)
      {
         // Set a viewport within the framebuffer.
         fb.setViewport(0, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a background color.
         fb.clearVP(java.awt.Color.black);
         model.doBackFaceCulling = false;
         model.facesHaveTwoSides = false;
         // Render again.
         renderer.renderScene(scene, fb);

         // Set a viewport within the framebuffer.
         fb.setViewport(400, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a background color.
         fb.clearVP(java.awt.Color.black);
         model.doBackFaceCulling = true;
         model.facesHaveTwoSides = false;
         // Render again.
         renderer.renderScene(scene, fb);

         // Set a viewport within the framebuffer.
         fb.setViewport(800, 0, 400, 400); //upper-left-hand-corner, width, height
         // Give the viewport a background color.
         fb.clearVP(java.awt.Color.black);
         model.doBackFaceCulling = false;
         model.facesHaveTwoSides = true;
         // Render again.
         renderer.renderScene(scene, fb);

         // save the framebuffer
         fb.dumpFB2File(String.format("PPM_Triangle_Frame%03d.ppm", i));

         // Rotate the model by 5 degrees.
         scene.camera.viewRotate(5, 0, 1, 0);
      }
   }
}