/*

*/
import java.awt.Color;

public class MainCylinderSection
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainCylinderSection.class.getName());
         System.exit(-1);;
      }

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the camera's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      //System.err.println( scene.toString() );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      double height = 4.0;

      int numberOfFrames = 72;
      for (int i = 0; i <= numberOfFrames; i++)
      {
         Model m = new CylinderSectionModel(k, n, 1, height,
                               2*Math.PI-(i*360.0/numberOfFrames)*(Math.PI/180), 2*Math.PI);

         m.doBackFaceCulling = false;  // Important: try changing this.

         //m.setColor(Color.red);
         //m.setRandomColors();
         for (Primitive p : m.primitiveList)
         {
            for (Vertex v : p.vertexList)
            {
               v.setColor(Math.abs(v.x), Math.abs(v.y)/height, Math.abs(v.z));
            }
         }

         // Add the model to the Scene.
         scene.modelList.add(m);

         // Clear the framebuffer.
         fb.clearFB(java.awt.Color.lightGray);
         // Render our scene into the frame buffer.
         renderer.renderScene(scene, fb);
         // Save the resulting image in a file.
         fb.dumpFB2File( String.format("PPM_CylinderSection_Frame%03d.ppm", i) );

         // Remove the model from the scene.
         scene.modelList.remove(0);
      }

      for (int i = 0; i <= numberOfFrames; i++)
      {
         Model m = new CylinderSectionModel(k, n, 1, height,
                               0, 2*Math.PI-(i*360.0/numberOfFrames)*(Math.PI/180));

         m.doBackFaceCulling = false;  // Important: try changing this.
         m.modelRotate((360/numberOfFrames)*i, 1,0,0);

         //m.setColor(Color.red);
         //m.setRandomColors();
         for (Primitive p : m.primitiveList)
         {
            for (Vertex v : p.vertexList)
            {
               v.setColor(Math.abs(v.x), Math.abs(v.y)/height, Math.abs(v.z));
            }
         }

         // Add the model to the Scene.
         scene.modelList.add(m);

         // Clear the framebuffer.
         fb.clearFB(java.awt.Color.lightGray);
         // Render our scene into the frame buffer.
         renderer.renderScene(scene, fb);
         // Save the resulting image in a file.
         fb.dumpFB2File( String.format("PPM_CylinderSection_Frame%03d.ppm", numberOfFrames+1+i) );

         // Remove the model from the scene.
         scene.modelList.remove(0);
      }
   }
}