/*
   This client demonstrates the "nested model"
   feature added to this version of the renderer.

   Here is the "scene graph" for this example.

                     mainModel
                    /    |    \
                  /      |      \
             model1    model2  model3
                  \      |     /
                    \    |    /
                      \  |  /
                     sphereModel
*/
import java.awt.Color;

public class MainNestedModels
{
   public static void main(String[] args)
   {
      double sin2PIover3 = Math.sin(2*Math.PI/3);
      // Create a Model to hold the nested models.
      Model mainModel = new Model();
      mainModel.doBackFaceCulling = false;
      // Add a single triangle to the triangle list
      // of this model.
      mainModel.addPrimitive(new Triangle(new Vertex( 1,    0,           0),
                                          new Vertex(-0.5,  sin2PIover3, 0),
                                          new Vertex(-0.5, -sin2PIover3, 0)));
      mainModel.primitiveList.get(0).vertexList.get(0).setColor( Color.red );
      mainModel.primitiveList.get(0).vertexList.get(1).setColor( Color.green );
      mainModel.primitiveList.get(0).vertexList.get(2).setColor( Color.blue );


      Model sphere = new SphereModel(10, 10, 0.5);
      sphere.doBackFaceCulling = true;
      // give every triangle a random color
      for (Primitive t : sphere.primitiveList)
      {
         t.setColorRandom();
      }
      // give every vertex a random color
//    for (Triangle t : sphere.triangleList)
//    {
//       t.v[0].setColorRandom();
//       t.v[1].setColorRandom();
//       t.v[2].setColorRandom();
//    }
      // Give the vertices colors based on their coordinates.
//    for (Triangle t : sphere.triangleList)
//    {
//       t.v[0].setColor( Math.abs(t.v[0].x), Math.abs(t.v[0].y), Math.abs(t.v[0].z) );
//       t.v[1].setColor( Math.abs(t.v[1].x), Math.abs(t.v[1].y), Math.abs(t.v[1].z) );
//       t.v[2].setColor( Math.abs(t.v[2].x), Math.abs(t.v[2].y), Math.abs(t.v[2].z) );
//    }


      // Create three sub-models.
      Model m1 = new Model();
      Model m2 = new Model();
      Model m3 = new Model();

      // Give each of these sub-models the sphere as a sub-model.
      m1.addSubModel( sphere );
      m2.addSubModel( sphere );
      m3.addSubModel( sphere );

      // Put these three sub-models into the main model.
      mainModel.addSubModel( m1 );
      mainModel.addSubModel( m2 );
      mainModel.addSubModel( m3 );

      // Place the three sub-models at the
      // corners of the main model's triangle.
      m1.modelTranslate( 1.5,   0,               0);
      m2.modelTranslate(-0.75,  1.5*sin2PIover3, 0);
      m3.modelTranslate(-0.75, -1.5*sin2PIover3, 0);


      Model floor = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      floor.doBackFaceCulling = false;
      floor.modelTranslate(0, -4, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(mainModel);
      scene.modelList.add(floor);


      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the camera's view frustum.
      double fov    = 45.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);

      // Set up the camera's location and orientation.
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);

      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();


      for (int i = 0; i <= 72; i++)
      {
         sphere.modelRotate(10, 0,1,0);

         mainModel.model2Identity();
         mainModel.modelRotate(5*i, 0, 0, 1);
         mainModel.modelTranslate(2, 0, 0);
//         mainModel.modelRotate(5*i, 0, 1, 0);
//         mainModel.modelRotate(5*i, 1, 0, 0);
         // Render
         fb.clearFB(java.awt.Color.lightGray);
         renderer.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_NestedModels_Frame%02d.ppm", i));
      }
   }
}