/*
   This client demonstrates the "nested model"
   feature added to this version of the renderer.

   Here is the "scene graph" for this example.

                     mainModel
                    /    |    \
                  /      |      \
             model1    model2  model3
                  \      |     /
                    \    |    /
                      \  |  /
                     sphereModel
*/
import java.awt.Color;

public class MainNestedModels_v2
{
   public static void main(String[] args)
   {
      double sin2PIover3 = Math.sin(2*Math.PI/3);
      // Create a Model to hold the nested models.
      Model mainModel = new Model();
      mainModel.doLighting = false;
      mainModel.doBackFaceCulling = false;
      // Add a single triangle to the triangle list
      // of this model.
      mainModel.addTriangle(new Triangle(new Vertex( 1,    0,           0),
                                         new Vertex(-0.5,  sin2PIover3, 0),
                                         new Vertex(-0.5, -sin2PIover3, 0)));
      mainModel.triangleList.getFirst().v[0].setColor( Color.red );
      mainModel.triangleList.getFirst().v[1].setColor( Color.green );
      mainModel.triangleList.getFirst().v[2].setColor( Color.blue );


      Model triangle = new Model();
      triangle.doLighting = false;
      triangle.doBackFaceCulling = false;
      triangle.addTriangle(new Triangle(new Vertex( 1,    0,           0),
                                        new Vertex(-0.5,  sin2PIover3, 0),
                                        new Vertex(-0.5, -sin2PIover3, 0)));
      triangle.triangleList.getFirst().v[0].setColor( Color.red );
      triangle.triangleList.getFirst().v[1].setColor( Color.red );
      triangle.triangleList.getFirst().v[2].setColor( Color.red );

      // Create three sub-models.
      Model m1 = new Model();
      Model m2 = new Model();
      Model m3 = new Model();

      // Give each of these sub-models the triangle as a sub-model.
      m1.addModel( triangle );
      m2.addModel( triangle );
      m3.addModel( triangle );

      // Put these three sub-models into the main model.
      mainModel.addModel( m1 );
      mainModel.addModel( m2 );
      mainModel.addModel( m3 );

      // Place the three sub-models at the
      // corners of the main model's triangle.
      m1.modelTranslate( 1.5,   0,               0);
      m2.modelTranslate(-0.75,  1.5*sin2PIover3, 0);
      m3.modelTranslate(-0.75, -1.5*sin2PIover3, 0);


      Model floor = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      floor.doLighting = false;
      floor.doBackFaceCulling = false;
      floor.modelTranslate(0, -3, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(mainModel);
      scene.modelList.add(floor);


      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 50.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);


      // Create a default (full) rendering pipeline.
      SceneRender renderer = new SceneRender(scene, fb);
      // Print out the pipeline.
      System.err.println( renderer );

      for (int i = 0; i <= 72; i++)
      {
//         m1.modelRotate(10, 1,0,0);

         mainModel.model2Identity();
         mainModel.modelRotate(5*i, 0, 0, 1);
         mainModel.modelTranslate(2, 0, 0);
         mainModel.modelRotate(5*i, 0, 1, 0);
//         mainModel.modelRotate(5*i, 1, 0, 0);
         // Render
         fb.clearFB(java.awt.Color.darkGray);
         renderer.renderScene();
         fb.dumpFB2File(String.format("PPM_NestedModels_Frame%02d.ppm", i));
      }
   }
}