/*

*/
import java.io.File;
import java.awt.Color;

public class MainObjSimple
{
   public static void main(String[] args)
   {
      // Create several Model objects.
      Model m1 = new ObjSimpleModel(new File("resources/horse.obj"));
      for (Primitive t : m1.primitiveList)
      {  // give every vertex a random color
         t.setColorRandom();
      }
      m1.doBackFaceCulling = true;
      m1.modelScale(0.5, 0.5, 0.5);
      m1.modelTranslate(-4, 0, -4);

      Model m2 = new ObjSimpleModel(new File("resources/small_rhombicosidodecahedron.obj"), true);
      m2.doBackFaceCulling = true;
      m2.modelScale(0.3, 0.3, 0.3);
      m2.modelTranslate(5, 0, 5);

      Model m3 = new ObjSimpleModel(new File("resources/great_rhombicosidodecahedron.obj"), true);
      m3.doBackFaceCulling = true;
      m3.modelScale(0.2, 0.2, 0.2);
      m3.modelTranslate(0, 0, 0);

      Model m4 = new ObjSimpleModel(new File("resources/apple.obj"), true);
      m4.doBackFaceCulling = true;
      m4.modelScale(2, 2, 2);
      m4.modelTranslate(0, 2, 0);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);

      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      // Render our scene into the frame buffer.
      renderer.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_ObjSimple_Frame%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         camera.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         renderer.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_ObjSimple_Frame%02d.ppm", i));
      }
   }
}