/*
   This version uses color to shade triangles.
   This version also tests converZB2FB().
*/
import java.awt.Color;

public class MainShapesColor
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainShapesColor.class.getName());
         System.exit(-1);;
      }

      // Create several Model objects.

      Model m2 = new CubeModel(1, 1, 1);
      for (Primitive t : m2.primitiveList)
         t.setColor( Color.cyan );
      m2.doBackFaceCulling = true;
      m2.modelTranslate(-4, 0, 0);

      Model m3 = new SphereModel(k, n, 1);
      // Give the vertices colors based on their coordinates.
      for (Primitive t : m3.primitiveList)
      {
         Vertex v0 = t.vertexList.get(0);
         Vertex v1 = t.vertexList.get(1);
         Vertex v2 = t.vertexList.get(2);
         v0.setColor( Math.abs(v0.x), Math.abs(v0.y), Math.abs(v0.z) );
         v1.setColor( Math.abs(v1.x), Math.abs(v1.y), Math.abs(v1.z) );
         v2.setColor( Math.abs(v2.x), Math.abs(v2.y), Math.abs(v2.z) );
      }
      m3.doBackFaceCulling = true;
      m3.modelTranslate(-2, 3, 0);

      Model m4 = new ConeModel(k, n, 1, 4);
      for (Primitive t : m4.primitiveList)
         t.setColor( Color.blue );
      m4.doBackFaceCulling = true;
      m4.modelTranslate(0, -2, 0);

      Model m5 = new CylinderModel(k, n, 1, 4);
      for (Primitive t : m5.primitiveList)
         t.setColor( Color.yellow );
      m5.doBackFaceCulling = true;
      m5.modelTranslate(3, 0, 0);

      Model m6 = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      m6.doBackFaceCulling = false;
      m6.modelTranslate(0, -5, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(m5);
      scene.modelList.add(m6);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);

      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      // Render our scene into the frame buffer.
      renderer.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_ShapesColor_Frame%02d.ppm", 0) );
      // Look at the contents of the z-buffer.
      FrameBuffer zb = fb.convertZB2FB();
      zb.dumpFB2File( String.format("PPM_ShapesColorZB_Frame%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         camera.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         renderer.renderScene(scene, fb);
         fb.dumpFB2File(String.format("PPM_ShapesColor_Frame%02d.ppm", i));
         zb = fb.convertZB2FB();
         zb.dumpFB2File( String.format("PPM_ShapesColorZB_Frame%02d.ppm", i) );
      }
   }
}