/*

*/
import java.awt.Color;

public class MainSphereSection2
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainSphereSection2.class.getName());
         System.exit(-1);;
      }


      Model m = new SphereSectionModel(k, n, 1, 5*Math.PI/8, 11*Math.PI/8, Math.PI/4, 3*Math.PI/4 );
      m.doBackFaceCulling = false;  // Important: try changing this.
      m.modelScale(4, 4, 4);
      m.modelRotate(90, 0,1,0);

      m.setColor(Color.red);
      m.setRandomColors();

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the model to the Scene.
      scene.modelList.add(m);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      camera.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      camera.viewLookAt(eyex,    eyey,    eyez,
                        centerx, centery, centerz,
                        upx,     upy,     upz);

      // Add the Camera to the Scene.
      scene.setCamera(camera);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);

      // Create a default rendering pipeline.
      SceneRender renderer = new SceneRender();

      int numberOfFrames = 72;
      for (int i = 0; i <= numberOfFrames; i++)
      {
         // Clear the framebuffer.
         fb.clearFB(java.awt.Color.lightGray);
         // Render our scene into the frame buffer.
         renderer.renderScene(scene, fb);
         // Save the resulting image in a file.
         fb.dumpFB2File( String.format("PPM_SphereSection2_Frame%03d.ppm", i) );
         // rotate the model
         m.modelRotate(360/numberOfFrames, 0,1,0);
      }
   }
}