/**
   An instance of this class represents one fragment of a rasterized triangle.

   Each fragment represents a potential pixel in the framebuffer (the xIndex and yIndex fields in the fragment). A fragment becomes a pixel if the fragment is in front of all the other fragments that would want to fill in that pixel (i.e., have the same values for xIndex and yIndex). The zCoord field in the fragment is used to detemine if a fragment is in front of (or behind) some other fragment.

   A fragment also know its location within the triangle it was rasterized from. This location is given by the alpha, beta, and gamma fields, (the fragments barycentric coordinates within the triangle).
*/

public class Fragment
{
   public int xIndex;        // x coordinate in the viewport.
   public int yIndex;        // y coordinate in the viewport.
   public double zCoord;     // Needed for z-buffer depth testing.
   public double alpha,      // The barycentric coordinates, with
                 beta,       // respect to this fragment's triangle,
                 gamma;      // of the center of this fragment.
   public double r, g, b, a; // Color of this fragment (including transparency)
}