/**
   Convert vertices from model coordinates to world coordinates.

   For each Model in a Scene, walk the Model's list of Primitive objects.
   For each Vertex object in each Primitive object, use the Model's model
   matrix to transform the Vertex object from model coordinates to world
   coordinates.
*/
import java.util.ListIterator;

public class P1_Model2World extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P1_Model2World(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each geometric primitive in the model, transform each vertex
      in the primitive from model coordinates to world coordinates.
   */
   public Model processModel(Model model)
   {
      Matrix modelMatrix = model.modelMatrix;

      // Transform each geometric primitive in the model's primitive list.
      for (Primitive p : model.primitiveList)
      {
         // Transform each vertex in the primitive.
         ListIterator<Vertex> itr = p.vertexList.listIterator();
         while (itr.hasNext())
         {
            Vertex v = itr.next();
            itr.set( modelMatrix.times(v) );
         }
      }
      return model;
   }
}