/**
   Convert line segments from world coordinates to view (camera) coordinates.

   For each Model in a Scene, walk the Model's list of LineSegment objects.
   For each Vertex object in each LineSgement object, use the Scene's view
   matrix to transform the Vertex object from world coordinates to view
   coordinates.
*/
import java.util.ListIterator;

public class P2_World2View extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P2_World2View(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each geometric primitive in the model, transform each vertex
      in the primitive from world coordinates to view coordinates.
   */
   public Model processModel(Model model)
   {
      Matrix viewMatrix = this.sceneRender.scene.camera.viewMatrix;

      // Transform each geometric primitive in the model's primitive list.
      for (Primitive p : model.primitiveList)
      {
         // Transform each vertex in the primitive.
         ListIterator<Vertex> itr = p.vertexList.listIterator();
         while (itr.hasNext())
         {
            Vertex v = itr.next();
            itr.set( viewMatrix.times(v) );
         }
      }
      return model;
   }
}