/**
   Convert vertices from view coordinates to clip coordinates
   relative to the standard clipping volume.

   For each Model in a Scene, walk the Model's list of Primitive objects.
   For each Vertex object in each Primitive object, use the Camera's
   projection matrix to project the Vertex object into the standard
   clipping volume. (The "standard clipping volume" is kind of a cube
   in the 4-dimensional projective space.)
*/
import java.util.ListIterator;

public class P4_View2Clip extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P4_View2Clip(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each geometric primitive in the model, transform each vertex
      in the primitive from view coordinates to clip coordinates.
   */
   public Model processModel(Model model)
   {
      Matrix projMatrix = this.sceneRender.scene.camera.projMatrix;

      // Transform each geometric primitive in the model's primitive list.
      for (Primitive p : model.primitiveList)
      {
         // Transform each vertex in the primitive.
         ListIterator<Vertex> itr = p.vertexList.listIterator();
         while (itr.hasNext())
         {
            Vertex v = itr.next();
            itr.set( projMatrix.times(v) );
         }
      }
      return model;
   }
}