/**
   Do the perspective division step of the pipeline.
*/
import java.util.ListIterator;

public class P7_Clip2NDC extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P7_Clip2NDC(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each geometric primitive in the model, transform each vertex
      in the primitive by dividing each of the coordinates in the vertex
      by the vertex's w coordinate, i.e.,
      (x, y, z, w) -> (x/w, y/w, z/w, 1).
   */
   public Model processModel(Model model)
   {
      for (Primitive p : model.primitiveList)
      {
         // Transform each vertex in the primitive.
         ListIterator<Vertex> itr = p.vertexList.listIterator();
         while (itr.hasNext())
         {
            // Do the perspective division.
            Vertex v = itr.next();
            itr.set( v.times(1.0/(v.w)) );
         }
      }
      return model;
   }
}