/**
   A positive x, y, and z axis in 3-dimensional space.
*/
import java.awt.Color;

public class Axes3DModel extends Model
{
   /**
      End points for the x, y, and z axes.
      Default color of black.
   */
   public Axes3DModel(double xMax, double yMax, double zMax)
   {
      this(xMax, yMax, zMax, Color.black, Color.black, Color.black);
   }


   /**
      End points for the x, y, and z axes.
      Use the given Color c.
   */
   public Axes3DModel(double xMax, double yMax, double zMax, Color c)
   {
      this(xMax, yMax, zMax, c, c, c);
   }


   /**
      End points for the x, y, and z axes.
      Use the colors cX, cY, and cZ for the x, y, and z axes.
   */
   public Axes3DModel(double xMax, double yMax, double zMax,
                      Color cX,    Color cY,    Color cZ)
   {
      super();

      Primitive xAxis = new LineSegment(new Vertex(0,    0,   0),
                                        new Vertex(xMax, 0,   0));
      Primitive yAxis = new LineSegment(new Vertex(0,    0,   0),
                                        new Vertex(0,   yMax, 0));
      Primitive zAxis = new LineSegment(new Vertex(0,    0,   0),
                                        new Vertex(0,    0,  zMax));
      xAxis.setColor( cX );
      yAxis.setColor( cY );
      zAxis.setColor( cZ );
      this.addPrimitive( xAxis );
      this.addPrimitive( yAxis );
      this.addPrimitive( zAxis );
   }
}