/**
                  v[4]
                   +-----------------+ v[5]
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |               y
           |       |         |       |               |
           |       |         |       |               |
           |  v[0] +---------|-------+ v[1]          |
           |      /          |      /                |
           |    /            |    /                  +----. x
           |  /              |  /                   /
           |/                |/                    /
           +-----------------+                    /
          v[3]              v[2]                 z

   xs the size of the cube along the x-axis
   ys the size of the cube along the y-axis
   zs the size of the cube along the z-axis
*/

public class CubeModel extends Model
{
   public CubeModel(double xs, double ys, double zs)
   {
      super();

      // Create the cube's geometry.
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(0,    0,    0);      // four vertices around the bottom face
      v[1] = new Vertex(0+xs, 0,    0);
      v[2] = new Vertex(0+xs, 0,    0+zs);
      v[3] = new Vertex(0,    0,    0+zs);
      v[4] = new Vertex(0,    0+ys, 0);      // four vertices around the top face
      v[5] = new Vertex(0+xs, 0+ys, 0);
      v[6] = new Vertex(0+xs, 0+ys, 0+zs);
      v[7] = new Vertex(0,    0+ys, 0+zs);

      // Create 12 triangles, 2 for each face of the cube.
      // bottom face
      this.addPrimitive( new Triangle(v[0], v[1], v[2]) );
      this.addPrimitive( new Triangle(v[2], v[3], v[0]) );
      // front face
      this.addPrimitive( new Triangle(v[3], v[2], v[6]) );
      this.addPrimitive( new Triangle(v[6], v[7], v[3]) );
      // right face
      this.addPrimitive( new Triangle(v[2], v[1], v[5]) );
      this.addPrimitive( new Triangle(v[5], v[6], v[2]) );
      // back face
      this.addPrimitive( new Triangle(v[1], v[0], v[4]) );
      this.addPrimitive( new Triangle(v[4], v[5], v[1]) );
      // left face
      this.addPrimitive( new Triangle(v[0], v[3], v[7]) );
      this.addPrimitive( new Triangle(v[7], v[4], v[0]) );
      // top face
      this.addPrimitive( new Triangle(v[7], v[6], v[5]) );
      this.addPrimitive( new Triangle(v[5], v[4], v[7]) );
   }
}//CubeModel