/**
   k number of triangles in each triangle fan at the top and bottom
   n number of triangle strips between the top and bottom triangle fans

   r1  radius of the base of the frustum
   r2  radius of the top of the frustum
   h   height of the frustum
*/

public class FrustumCircularModel extends Model
{
   public FrustumCircularModel(int k, int n, double r1, double r2, double h)
   {
      super();

      // Create the frustum's geometry.

      // An array of vertices to be used to create triangles.
      Vertex[][] v = new Vertex[n+1][k];

      // Create all the vertices along the cylinder wall.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            double slantRadius = (i/(double)(n)) * r1 + ((n-i)/(double)(n)) * r2;
            v[i][j] = new Vertex( slantRadius*c, h-(i*h)/n, slantRadius*s );
         }
      }

      // Vertex at the center of the top.
      Vertex    topCenter = new Vertex(0.0,   h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, 0.0, 0.0);


      // Create all the triangle strips around the cylinder wall.
      Triangle t;
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            t = new Triangle( v[i][j], v[i][j+1], v[i+1][j+1] );
            this.addPrimitive( t );
            t = new Triangle( v[i+1][j+1], v[i+1][j], v[i][j] );
            this.addPrimitive( t );
         }
         t = new Triangle( v[i][k-1], v[i][0], v[i+1][0] );
         this.addPrimitive( t );
         t = new Triangle( v[i+1][0], v[i+1][k-1], v[i][k-1] );
         this.addPrimitive( t );
      }


      // Create the triangle fan at the top.
      for (int j = 0; j < k-1; j++)
      {
         t = new Triangle( topCenter, v[0][j+1], v[0][j] );
         this.addPrimitive( t );
      }
      t = new Triangle( topCenter, v[0][0], v[0][k-1] );
      this.addPrimitive( t );


      // Create the triangle fan at the bottom.
      for (int j = 0; j < k-1; j++)
      {
         t = new Triangle( bottomCenter, v[n][j], v[n][j+1] );
         this.addPrimitive( t );
      }
      t = new Triangle( bottomCenter, v[n][k-1], v[n][0] );
      this.addPrimitive( t );
   }
}//CylinderModel