/**
   This can read a simple OBJ file. The file
   can have non-triangular faces, but normals
   and material properties are ignored.
*/
import java.util.regex.*;
import java.util.ArrayList;
import java.util.Scanner;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;

public class ObjSimpleModel extends Model
{
   public ObjSimpleModel(File modelFile)
   {
      this(modelFile, false);
   }

   public ObjSimpleModel(File modelFile, boolean colorFaces)
   {
      super();

      // Open the OBJ file.
      String objName = null;
      FileInputStream fis = null;
      try
      {
         objName = modelFile.getCanonicalPath();
         fis = new FileInputStream( modelFile );
      }
      catch (FileNotFoundException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open OBJ file: %s\n", objName);
         System.exit(-1);
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not open OBJ file: %s\n", objName);
         System.exit(-1);
      }

      // Get the geometry from the OBJ file.
      ArrayList<Vertex> verts = new ArrayList<Vertex>();
      try
      {
         // Pattern for parsing lines that start with "f"
         Pattern p = Pattern.compile("^(\\d*)[/]?(\\d*)[/]?(\\d*)");

         Scanner scanner = new Scanner(fis);
         while ( scanner.hasNext() )
         {
            String token = scanner.next();
            if ( token.startsWith("#") )
            {
               scanner.nextLine(); // skip over the comment
            }
            else if ( token.startsWith("v") )
            {
               double x = scanner.nextDouble();
               double y = scanner.nextDouble();
               double z = scanner.nextDouble();
               Vertex v = new Vertex(x, y, z);
               verts.add( v );
            }// parse vertex
            else if ( token.startsWith("f") )
            {
               // tokenize the rest of the line
               String restOfLine = scanner.nextLine();
               Scanner scanner2 = new Scanner( restOfLine );
               // parse three vertices and make a triangle
               int[] v = new int[3];
               for (int i = 0; i < 3; i++)
               {
                  // parse a "v/vt/vn" group
                  String faceGroup = scanner2.next();
                  Matcher m = p.matcher( faceGroup );
                  if ( m.find() )
                  {
                     v[i] = Integer.parseInt( m.group(1) );
                     String vt = m.group(2);  // don't need
                     String vn = m.group(3);  // don't need
                  }
                  else
                     System.err.println("Error: bad face: " + faceGroup);
               }
               if (colorFaces)
               {
                  verts.get(v[0]-1).setColorRandom();
                  verts.get(v[1]-1).setColor(verts.get(v[0]-1));
                  verts.get(v[2]-1).setColor(verts.get(v[0]-1));
               }
               Triangle t = new Triangle(verts.get(v[0]-1), verts.get(v[1]-1), verts.get(v[2]-1));
               this.addPrimitive( t );

               while (scanner2.hasNext())  // parse another vertex and make a triangle
               {
                  v[1] = v[2];
                  String faceGroup = scanner2.next();
                  Matcher m = p.matcher( faceGroup );
                  if ( m.find() )
                  {
                     v[2] = Integer.parseInt( m.group(1) );
                     String vt = m.group(2);  // don't need
                     String vn = m.group(3);  // don't need
                  }
                  else
                     System.err.println("Error: bad face: " + faceGroup);

                  if (colorFaces)
                  {
                     verts.get(v[2]-1).setColor(verts.get(v[0]-1));
                  }
                  t = new Triangle(verts.get(v[0]-1), verts.get(v[1]-1), verts.get(v[2]-1));
                  this.addPrimitive( t );
               }
            }// parse face
         }// parse one line
         fis.close();
      }
      catch (IOException e)
      {
         e.printStackTrace(System.err);
         System.err.printf("ERROR! Could not read OBJ file: %s\n", objName);
         System.exit(-1);
      }
   }
}//ObjSimpleModel