/**
   This is a regular octahedron with side length 1.
   Its center is at the orgin and the center plane is the xz-plane.
   The 5th and 6th vertices are on the y-axis.

   http://en.wikipedia.org/wiki/Regular_octahedron
*/

public class OctahedronModel extends Model
{
   public OctahedronModel()
   {
      super();

      // Create the octahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 0.5, 0,  0.5);  // four vertices around the center plane
      v[1] = new Vertex(-0.5, 0,  0.5);
      v[2] = new Vertex(-0.5, 0, -0.5);
      v[3] = new Vertex( 0.5, 0, -0.5);
      v[4] = new Vertex( 0,  1/sqrt2, 0);   // vertex at the top
      v[5] = new Vertex( 0, -1/sqrt2, 0);   // vertex at the bottom

      // Create 8 triangles.
      // face 1
      this.addPrimitive( new Triangle(v[0], v[1], v[4]) );
      // face 2
      this.addPrimitive( new Triangle(v[1], v[2], v[4]) );
      // face 3
      this.addPrimitive( new Triangle(v[2], v[3], v[4]) );
      // face 4
      this.addPrimitive( new Triangle(v[3], v[0], v[4]) );

      // face 5
      this.addPrimitive( new Triangle(v[1], v[0], v[5]) );
      // face 6
      this.addPrimitive( new Triangle(v[2], v[1], v[5]) );
      // face 7
      this.addPrimitive( new Triangle(v[3], v[2], v[5]) );
      // face 8
      this.addPrimitive( new Triangle(v[0], v[3], v[5]) );
   }
}//OctahedronModel