/**
   A flat checkerboard in the xz-plane.
*/
import java.awt.Color;

public class PanelModel extends Model
{
   /**
      A checkerboard without colors.
   */
   public PanelModel(int xMin, int xMax, int zMin, int zMax)
   {
      this(xMin, xMax, zMin, zMax, null, null);
   }

   /**
      A checkerboard with two colors.
   */
   public PanelModel(int xMin, int xMax, int zMin, int zMax, Color c1, Color c2)
   {
      super();

      // Create the checkerboard's geometry.
      Vertex[][] v = new Vertex[(xMax-xMin)+1][(zMax-zMin)+1];

      for (int x = xMin; x <= xMax; x++)
         for (int z = zMin; z <= zMax; z++)
         {
            v[x-xMin][z-zMin] = new Vertex(x, 0, z);
         }

      // Create the triangles and give each triangle a color.
      for (int x = 0; x < xMax-xMin; x++)
      {
         for (int z = 0; z < zMax-zMin; z++)
         {
            Triangle t1 = new Triangle( v[x][z],     v[x][z+1], v[x+1][z+1] );
            Triangle t2 = new Triangle( v[x+1][z+1], v[x+1][z], v[x][z] );
            if ( (x+z)%2 == 0 )
            {
               if (null != c1)
               {
                  t1.setColor( c1 );
                  t2.setColor( c1 );
               }
            }
            else  // use the other color
            {
               if (null != c2)
               {
                  t1.setColor( c2 );
                  t2.setColor( c2 );
               }
            }
            this.addPrimitive( t1 );
            this.addPrimitive( t2 );
         }
      }
   }
}