/**
   This is a prism with side length 1, centered at the origin,
   and with its centeral axis on the y-axis.
   The total height is 2*(h + h2).
*/

public class PrismModel extends Model
{
   /**
      A prism with a regular tetrahedron at the top and bottom.
   */
   public PrismModel(double h)
   {
      this(h, (float)(Math.sqrt(2)/Math.sqrt(3)) );
   }


   /**
      A prism with a tetrahedron at the top and bottom, where
      the face-edge-face angle of the tetrahedron is theta.
      theta is in radians!
      (If theta = arctan(2*sqrt(2)), then the tetrathedron is regular.)
   */
   public PrismModel(double h, double theta)
   {
      this(h, (float)(Math.tan(theta)/(2*Math.sqrt(3))) );
   }


   /**
      A prism with a tetrahedron of height h2 at the top and bottom.
   */
   private PrismModel(double h, float h2)
   {
      super();

      // Create the prism's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[8];

      v[0] = new Vertex( 1.0/sqrt3, -h,  0);    // three vertices around the bottom
      v[1] = new Vertex(-0.5/sqrt3, -h,  0.5);
      v[2] = new Vertex(-0.5/sqrt3, -h, -0.5);

      v[3] = new Vertex( 1.0/sqrt3,  h,  0);    // three vertices around the top
      v[4] = new Vertex(-0.5/sqrt3,  h,  0.5);
      v[5] = new Vertex(-0.5/sqrt3,  h, -0.5);

      v[6] = new Vertex( 0, -h-h2, 0);  // vertex at the bottom
      v[7] = new Vertex( 0,  h+h2, 0);  // vertex at the top

      // Create 12 triangles, 2 for each face of the prism,
      // 3 triangles at the top, and 3 triangles at the bottom
      // 3 bottom faces
      this.addPrimitive( new Triangle(v[6], v[0], v[1]) );
      this.addPrimitive( new Triangle(v[6], v[1], v[2]) );
      this.addPrimitive( new Triangle(v[6], v[2], v[0]) );
      // vertical face 1
      this.addPrimitive( new Triangle(v[0], v[1], v[4]) );
      this.addPrimitive( new Triangle(v[4], v[3], v[0]) );
      // vertical face 2
      this.addPrimitive( new Triangle(v[1], v[2], v[5]) );
      this.addPrimitive( new Triangle(v[5], v[4], v[1]) );
      // vertical face 3
      this.addPrimitive( new Triangle(v[2], v[0], v[3]) );
      this.addPrimitive( new Triangle(v[3], v[5], v[2]) );
      // 3 top faces
      this.addPrimitive( new Triangle(v[7], v[3], v[4]) );
      this.addPrimitive( new Triangle(v[7], v[4], v[5]) );
      this.addPrimitive( new Triangle(v[7], v[5], v[3]) );
   }
}