/**
   k number of triangles in each triangle fan at the top and the bottom
   n number of triangle strips between the top and bottom triangle fans

   r radius of the sphere
*/

public class SphereModel extends Model
{
   public SphereModel(int k, int n, double radius)
   {
      super();

      // Create the sphere's geometry.

      // An array of vertices to be used to create triangles.
      Vertex[][] v = new Vertex[n+1][k];

      // Create all the vertices.
      for (int j = 0; j < k; j++)
      {
         double c1 = Math.cos(j*(2.0*Math.PI)/k);
         double s1 = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            double c2 = Math.cos((i+1)*Math.PI/(n+2));
            double s2 = Math.sin((i+1)*Math.PI/(n+2));
            v[i][j] = new Vertex( radius*s2*c1, radius*c2, radius*s2*s1 );
         }
      }
      // Vertex at the center of the top.
      Vertex    topPole = new Vertex(0.0,  radius, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomPole = new Vertex(0.0, -radius, 0.0);

      // Create the triangle fan at the top.
      Triangle t;
      for (int j = 0; j < k-1; j++)
      {
         t = new Triangle( topPole, v[0][j+1], v[0][j] );
         this.addPrimitive( t );
      }
      t = new Triangle( topPole, v[0][0], v[0][k-1] );
      this.addPrimitive( t );


      // Create all the triangle strips between the top and bottom triangle fans.
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            t = new Triangle( v[i][j], v[i][j+1], v[i+1][j+1] );
            this.addPrimitive( t );
            t = new Triangle( v[i+1][j+1], v[i+1][j], v[i][j] );
            this.addPrimitive( t );
         }
         t = new Triangle( v[i][k-1], v[i][0], v[i+1][0] );
         this.addPrimitive( t );
         t = new Triangle( v[i+1][0], v[i+1][k-1], v[i][k-1] );
         this.addPrimitive( t );
      }


      // Create the triangle fan at the bottom.
      for (int j = 0; j < k-1; j++)
      {
         t = new Triangle( bottomPole, v[n][j], v[n][j+1] );
         this.addPrimitive( t );
      }
      t = new Triangle( bottomPole, v[n][k-1], v[n][0] );
      this.addPrimitive( t );
   }
}//SphereModel